"""Linguo aims to make model translation easy and is designed to let you use the built-in Django features (Query API, Model Forms, Admin, etc) as intended."""

VERSION = (1, 3, 2)

__version__ = '.'.join(map(str, VERSION))
